@@ -331,7 +331,6 @@ e_gw_connection_new (const char *uri, const char *username, const char *password
 	if (param) {
 		SoupSoapParameter *subparam;
 		const char *param_value;
-		int i;
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "email");
 		if (subparam) {
@@ -545,7 +544,7 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
 		EGwItem *item;
 
-		item = e_gw_item_new_from_soap_parameter (container, subparam);
+		item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, subparam);
 		if (item)
 			*list = g_list_append (*list, item);
         }
@@ -614,7 +613,7 @@ e_gw_connection_get_items_from_ids (EGwConnection *cnc, const char *container, c
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
 		EGwItem *item;
 
-		item = e_gw_item_new_from_soap_parameter (container, subparam);
+		item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, subparam);
 		if (item)
 			*list = g_list_append (*list, item);
         }
@@ -723,7 +722,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
 				/*process each item */  
 				EGwItem *item;
 				/*FIXME  pass the container id */
-				item = e_gw_item_new_from_soap_parameter ("Calendar", subparam);
+				item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, "Calendar", subparam);
                                 if (!item) { 
                                          g_object_unref (response); 
                                          g_object_unref (msg); 
@@ -745,7 +744,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
 			 EGwItem *item;
 			 /*process each item */
 			 /*item = get_item_from_updates (subparam);*/
-			 item = e_gw_item_new_from_soap_parameter ("Calendar", subparam);
+			 item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, "Calendar", subparam);
 			 if (item)
 				 *updates = g_slist_append (*updates, item);
                  } 
@@ -942,7 +941,7 @@ e_gw_connection_get_item (EGwConnection *cnc, const char *container, const char
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 	
-       	*item = e_gw_item_new_from_soap_parameter (container, param);
+       	*item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, param);
 	
                
 	/* free memory */
@@ -1036,8 +1035,8 @@ e_gw_connection_accept_request (EGwConnection *cnc, const char *id, const char *
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "acceptRequest");
 	soup_soap_message_start_element (msg, "items", NULL, NULL);
 	e_gw_message_write_string_parameter (msg, "item", NULL, id);
-	e_gw_message_write_string_parameter (msg, "acceptLevel", NULL, accept_level);
 	soup_soap_message_end_element (msg);
+	e_gw_message_write_string_parameter (msg, "acceptLevel", NULL, accept_level);
 	e_gw_message_write_footer (msg);
 
 	response = e_gw_connection_send_message (cnc, msg);
@@ -1408,7 +1407,7 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
         SoupSoapResponse *response;
         EGwConnectionStatus status;
         SoupSoapParameter *param, *subparam, *second_level_child;
-	const char *id, *name;
+	char *id, *name;
         g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 	
 	/* build the SOAP message */
@@ -1557,3 +1556,148 @@ e_gw_connection_remove_members (EGwConnection *cnc, const char *group_id, GList
 
 
 }
+
+
+EGwConnectionStatus
+e_gw_connection_create_cursor (EGwConnection *cnc, const char *container, const char *view, EGwFilter *filter, int *cursor)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+        EGwConnectionStatus status;
+	SoupSoapParameter *param;
+	char *value;
+
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail ((container != NULL), E_GW_CONNECTION_STATUS_UNKNOWN);
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "createCursorRequest");
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
+	if (view)
+		e_gw_message_write_string_parameter (msg, "view", NULL, view);
+	if (E_IS_GW_FILTER(filter))
+		e_gw_filter_append_to_soap_message (filter, msg);
+	
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	
+	status = e_gw_connection_parse_response_status (response);
+        if (status != E_GW_CONNECTION_STATUS_OK) {
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	}
+	param = soup_soap_response_get_first_parameter_by_name (response, "cursor");
+	if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	value = soup_soap_parameter_get_string_value(param);
+	
+	if (!value) {
+		 g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+	*cursor =(int) g_ascii_strtod (value, NULL);
+	
+	g_free (value);
+	g_object_unref (response);
+	g_object_unref (msg);
+        return E_GW_CONNECTION_STATUS_OK;
+}
+
+EGwConnectionStatus
+e_gw_connection_destroy_cursor (EGwConnection *cnc, const char *container,  int cursor)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+        EGwConnectionStatus status;
+	
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail ((container != NULL), E_GW_CONNECTION_STATUS_UNKNOWN);
+	
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "destroyCursorRequest");
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
+
+	e_gw_message_write_int_parameter (msg, "cursor", NULL, cursor);
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	
+	status = e_gw_connection_parse_response_status (response);
+	
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+     
+}
+
+
+EGwConnectionStatus
+e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int cursor, int forward, int count, GList **item_list)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+        EGwConnectionStatus status;
+	SoupSoapParameter *param, *subparam;
+	
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail ((container != NULL), E_GW_CONNECTION_STATUS_UNKNOWN);
+	
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "readCursorRequest");
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
+	e_gw_message_write_int_parameter (msg, "cursor", NULL, cursor);
+	/* there is problem in read curosr if you set this, uncomment after the problem 
+	   is fixed in server */
+	//	e_gw_message_write_int_parameter (msg, "forward", NULL, forward);
+	e_gw_message_write_int_parameter (msg, "count", NULL, count);
+	
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	
+	status = e_gw_connection_parse_response_status (response);
+        if (status != E_GW_CONNECTION_STATUS_OK) {
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	}
+
+	/* if status is OK - parse result. return the list */	
+	param = soup_soap_response_get_first_parameter_by_name (response, "items");
+        if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+	for (subparam = soup_soap_parameter_get_first_child_by_name (param, "item");
+	     subparam != NULL;
+	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
+		EGwItem *item;
+
+		item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, subparam);
+		if (item)
+			*item_list = g_list_append (*item_list, item);
+        }
+               
+	/* free memory */
+        g_object_unref (response);
+	g_object_unref (msg);
+        return E_GW_CONNECTION_STATUS_OK;
+}
